// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: cat_manager.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __CAT_MANAGER_H
#define __CAT_MANAGER_H

#include "catmap_util.h"

#include <algorithm>
#include <iosfwd>
#include <map>
#include <string>
#include <vector>


namespace SITE_CAT_TYPE {
  enum index_t {
    MUT_RATE,
    MUT_MODEL,
    SEL_STRENGTH,
    SEL_MATRIX,
    ROOT,
    SIZE};
}

// persistent options:
struct cat_manager_options_base {
  cat_manager_options_base()
    : data_catmap() {}

  data_catmap_type data_catmap;
};

// non-persistent options
struct cat_manager_options : public cat_manager_options_base {
  cat_manager_options()
    : cat_manager_options_base(),
      n_site_cats(SITE_CAT_TYPE::SIZE,1) {}

  std::vector<unsigned> n_site_cats;
};


struct cat_manager{

  typedef cat_manager self_t;

  explicit
  cat_manager(std::istream& is);

  explicit
  cat_manager(const cat_manager_options& cmo);

private:
  self_t& operator=(const self_t& rhs); // disable copy
public:

  unsigned
  get_typed_cat_no(const SITE_CAT_TYPE::index_t sct,
                   const unsigned site_cat) const {
    return cat_index[sct][site_cat];
  }

  unsigned
  get_typed_cat_size(const SITE_CAT_TYPE::index_t sct) const {
    return 1+*(std::max_element(cat_index[sct].begin(),cat_index[sct].end()));
  }

  unsigned
  get_cat_size() const {
    return cat_index[0].size();
  }

  unsigned
  get_assigned_cat_size() const {
    return assigned_cat_size(_opt.data_catmap);
  }

  bool
  is_unassigned_cat() const {
    return ::is_unassigned_cat(_opt.data_catmap);
  }

  void store_state(std::ostream& os) const;

  const data_catmap_type& data_catmap() const { return _opt.data_catmap; }

private:
  /////// data:
  typedef std::vector<std::vector<unsigned> > uvec_t;

  cat_manager_options_base _opt;
  uvec_t cat_index;
};


#endif
