// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: cat_post_prob.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "bi_tree_type.h"
#include "cat_post_prob.h"
#include "site_data_fastlup.h"
#include "subs_ml_types.h"

#include <iostream>
#include <vector>


void
cat_post_prob_report(const RATE_GTOR_MODEL::index_t rgm,
                     const bi_tree_sml_type& tree,
                     const int n_site_cats,
                     const site_data_fastlup& full_sdf,
                     const prob_t* cat_site_prob,            // n_site_cats*full_sdf.len
                     const std::vector<prob_t>& site_cat_prior_pdistro){

  assert(full_sdf.n_orgs == tree.n_leaves());

  const SITE_MODEL::index_t sm(RATE_GTOR_MODEL::convert_to_site_model(rgm));
  const unsigned n_leaves(tree.n_leaves());

  std::ostream& os(std::cout);

  // first write out site info:
  os << "base_size: " << SITE_MODEL::base_size(sm) << "\n";
  os << "repeat_size: " << RATE_GTOR_MODEL::base_size_conditioned(rgm) << "\n";
  os << "repeat_offset: " << RATE_GTOR_MODEL::base_size_repeat_offset(rgm) << "\n";

  // write out org id's:
  for(unsigned j(0);j<n_leaves;++j){
    if(j) os << " ";
    os << tree.leaf_label(j);
  }
  os << "\n";

  std::vector<smlfloat> pp(n_site_cats);
  for(unsigned i(0);i<full_sdf.len;++i){
    if(full_sdf.data[i].unassigned_cat_count == 0) continue;

    for(unsigned j(0);j<n_leaves;++j){
      if(j) os << " ";
      os << SITE_MODEL::print(sm,full_sdf.data_core[i].index[j]);
    }

    smlfloat sum(0.);
    for(int sc(0);sc<n_site_cats;++sc){
      pp[sc] = cat_site_prob[i+sc*full_sdf.len]*site_cat_prior_pdistro[sc];
      sum += pp[sc];
      //os << pp[sc] << " ";
    }
    //os << sum;

    for(int sc(0);sc<n_site_cats;++sc) os << " " << pp[sc]/sum;
    os << "\n";
  }
}
