// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: nuc_seq_data.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __NUC_SEQ_DATA_H
#define __NUC_SEQ_DATA_H


#include "name_id_lup.h"
#include "simple_util.h"
#include "util/bio/bioseq_util.h"

#include <map>
#include <string>
#include <vector>


struct align_col_t {

  typedef unsigned char cat_type;

  align_col_t() : cat_no(0), codon_pos(0), is_continuous(true) {}

  unsigned char cat_no;
  unsigned char codon_pos;
  bool is_continuous;       ///< is this column continuous with the preceding column?
};


struct align_dat_t {
  typedef std::vector<align_col_t> info_type;
  typedef simple_init_matrix<NUC::index_t> seq_type;

  std::string label;
  seq_type seq; // n_seqs*ncol
  info_type info;
};



/// \brief intermediate data format between seq file/dir read and
/// conversion to independent site data exchange format
///
/// this is intended to be an abstraction of sequence and category
/// data from any file format, to be used as a common entry point for
/// all downstream sequence processing.
///
struct nuc_seq_data {

  nuc_seq_data(std::string* cmd=0) : cmdline_ptr(cmd) {}

  unsigned n_seqs() const { return taxid.size(); }

  unsigned max_cat() const;

  void load_state(std::istream& is);

  void store_state(std::ostream& os) const;

  std::map<unsigned,std::string> id_2_catlabel;
  name_id_lup taxid;
  std::vector<align_dat_t> dat;

  std::string* cmdline_ptr;
};


void
nuc_seq_data_report(const nuc_seq_data& in_seq,
                    std::ostream& os);

#endif
