// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: prob_gtor_state_conversion.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __PROB_GTOR_STATE_CONVERSION_H
#define __PROB_GTOR_STATE_CONVERSION_H

#include "prob_gtor.h"
#include "rate_edge_dependencies.h"
#include "util/bio/bioseq_util.h"
#include "util/bio/bioseq_util_pdf_conversions.h"

#if 0
/// generalization of the distro's below -- in progress
///
template <typename FloatType>
struct prob_gtor_bio_pdistro_builder : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_bio_pdistro_builder<FloatType> self_t;

  prob_gtor_bio_pdistro_builder(const SITE_MODEL::index_t target,
                                const BIO_PDISTRO::index_t param)
    : base_t(BIO_PDISTRO::state_size(BIO_PDISTRO::convert_from_site_model(target)),
             BIO_PDISTRO::state_size(param)) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    bio_pdistro_convert(BIO_PDISTRO::convert_from_site_model(target),param,this->_distro,x);
  }

  virtual
  void param_update(const bool) {
    const unsigned ps(param_size());
    FloatType* x2(new FloatType[ps]);
    bio_pdistro_norm(param,x,x2);
    bio_pdistro_convert(param,BIO_PDISTRO::convert_from_site_model(target),x2,this->_distro);
    delete [] x2;
  }
};
#endif

///
template <typename FloatType>
struct prob_gtor_dinuc_from_nuc : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_dinuc_from_nuc<FloatType> self_t;

  prob_gtor_dinuc_from_nuc() : base_t(DINUC::SIZE,PARAM_SIZE) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    dinuc_pdf_2_nuc_pdf_pos1(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+PARAM_SIZE,
                            this->_is_train_param.begin());
    nuc_pdf_2_dinuc_pdf(this->_param,this->_distro);
  }

  enum { PARAM_SIZE = NUC::SIZE };
};




/// \brief nsc4 distro from nscodon distro params plus the n2,n0
/// codon boundary joint nuc distro
///
template <typename FloatType>
struct prob_gtor_nsc4_from_nscodon_dinuc : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nsc4_from_nscodon_dinuc<FloatType> self_t;

  prob_gtor_nsc4_from_nscodon_dinuc(const bool is_c4_pre)
    : base_t(NSC4::SIZE,NSCODON::SIZE+DINUC::SIZE), _is_c4_pre(is_c4_pre) {
    this->_is_free_block_stop[NSCODON::SIZE-1] = true;
    this->_is_free_block_start[NSCODON::SIZE] = true;
  }

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nsc4_pdf_2_nscodon_pdf(this->_distro,this->_param);
    nsc4_pdf_2_codon_boundary_dinuc_pdf(this->_distro,_is_c4_pre,this->_param+NSCODON::SIZE);

    shape_dinuc_nscodon(this->_param,this->_param+NSCODON::SIZE);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+NSCODON::SIZE,
                            this->_is_train_param.begin());

    pdistro_norm_free_param(this->_param+NSCODON::SIZE,
                            this->_param+NSCODON::SIZE+DINUC::SIZE,
                            this->_is_train_param.begin()+NSCODON::SIZE);

    if(_is_c4_pre){
      nscodon_pdf_dinuc_pdf_2_nsc4_pre_pdf(this->_param,this->_param+NSCODON::SIZE,this->_distro);
    } else {
      nscodon_pdf_dinuc_pdf_2_nsc4_post_pdf(this->_param,this->_param+NSCODON::SIZE,this->_distro);
    }
  }

  bool _is_c4_pre;
};




/// \brief nsc4 distro built from nscodon parameters
///
template <typename FloatType>
struct prob_gtor_nsc4_from_nscodon : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nsc4_from_nscodon<FloatType> self_t;

  prob_gtor_nsc4_from_nscodon(const bool is_c4_pre)
    : base_t(NSC4::SIZE,PARAM_SIZE), _is_c4_pre(is_c4_pre) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nsc4_pdf_2_nscodon_pdf(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+PARAM_SIZE,
                            this->_is_train_param.begin());
    if(_is_c4_pre){
      nscodon_pdf_2_nsc4_pre_pdf(this->_param,this->_distro);
    } else {
      nscodon_pdf_2_nsc4_post_pdf(this->_param,this->_distro);
    }
  }

  enum { PARAM_SIZE = NSCODON::SIZE };

  bool _is_c4_pre;
};




/// \brief nsc4 distro build from positional nuc params
///
template <typename FloatType>
struct prob_gtor_nsc4_from_nuc_pos : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nsc4_from_nuc_pos<FloatType> self_t;

  prob_gtor_nsc4_from_nuc_pos(const bool is_c4_pre)
    : base_t(NSC4::SIZE,PARAM_SIZE), _is_c4_pre(is_c4_pre) {
    for(unsigned i(1);i<3;++i){
      this->_is_free_block_stop[i*NUC::SIZE-1] = true;
      this->_is_free_block_start[i*NUC::SIZE] = true;
    }
  }

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nsc4_pdf_2_nuc_pdf_all_pos(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    for(unsigned i(0);i<CODON::BASE_SIZE;++i){
      pdistro_norm_free_param(this->_param+NUC::SIZE*i,
                              this->_param+NUC::SIZE*(i+1),
                              this->_is_train_param.begin()+NUC::SIZE*i);
    }
    if(_is_c4_pre){
      nuc_pdf_all_pos_2_nsc4_pre_pdf(this->_param,this->_distro);
    } else {
      nuc_pdf_all_pos_2_nsc4_post_pdf(this->_param,this->_distro);
    }
  }

  enum { PARAM_SIZE = NUC::SIZE*CODON::BASE_SIZE };

  bool _is_c4_pre;
};




/// \brief nsc4 distro (pre or post) build from nuc params
///
template <typename FloatType>
struct prob_gtor_nsc4_from_nuc : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nsc4_from_nuc<FloatType> self_t;

  prob_gtor_nsc4_from_nuc() : base_t(NSC4::SIZE,PARAM_SIZE) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nsc4_pdf_2_nuc_pdf(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+PARAM_SIZE,
                            this->_is_train_param.begin());
    nuc_pdf_2_nsc4_pdf(this->_param,this->_distro);
  }

  enum { PARAM_SIZE = NUC::SIZE };
};




/// \brief generate nsc5 distro from nscodon distro plus the n2,n0
/// codon boundary joint nucleotide distro
///
template <typename FloatType>
struct prob_gtor_nsc5_from_nscodon_dinuc : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nsc5_from_nscodon_dinuc<FloatType> self_t;

  prob_gtor_nsc5_from_nscodon_dinuc() : base_t(NSC5::SIZE,NSCODON::SIZE+DINUC::SIZE) {
    this->_is_free_block_stop[NSCODON::SIZE-1] = true;
    this->_is_free_block_start[NSCODON::SIZE] = true;
  }

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nsc5_pdf_2_nscodon_pdf(this->_distro,this->_param);
    nsc5_pdf_2_codon_boundary_dinuc_pdf(this->_distro,this->_param+NSCODON::SIZE);

    shape_dinuc_nscodon(this->_param,this->_param+NSCODON::SIZE);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+NSCODON::SIZE,
                            this->_is_train_param.begin());

    pdistro_norm_free_param(this->_param+NSCODON::SIZE,
                            this->_param+NSCODON::SIZE+DINUC::SIZE,
                            this->_is_train_param.begin()+NSCODON::SIZE);

    nscodon_pdf_dinuc_pdf_2_nsc5_pdf(this->_param,
                                     this->_param+NSCODON::SIZE,
                                     this->_distro);
  }
};




/// \brief frequencies for nsc5 states build from
/// nscodon frequency parameters
///
template <typename FloatType>
struct prob_gtor_nsc5_from_nscodon : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nsc5_from_nscodon<FloatType> self_t;

  prob_gtor_nsc5_from_nscodon() : base_t(NSC5::SIZE,PARAM_SIZE) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nsc5_pdf_2_nscodon_pdf(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+PARAM_SIZE,
                            this->_is_train_param.begin());
    nscodon_pdf_2_nsc5_pdf(this->_param,this->_distro);
  }

  enum { PARAM_SIZE = NSCODON::SIZE };
};




/// \brief frequencies for nsc4 states (pre or post) build from
/// nscodon frequency parameters
///
template <typename FloatType>
struct prob_gtor_nscodon_from_nuc : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nscodon_from_nuc<FloatType> self_t;

  prob_gtor_nscodon_from_nuc() : base_t(NSCODON::SIZE,PARAM_SIZE) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nscodon_pdf_2_nuc_pdf(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+PARAM_SIZE,
                            this->_is_train_param.begin());
    nuc_pdf_2_nscodon_pdf(this->_param,this->_distro);
  }

  enum { PARAM_SIZE = NUC::SIZE };
};




/// \brief frequencies for nsc4 states (pre or post) build from
/// nscodon frequency parameters
///
template <typename FloatType>
struct prob_gtor_nscodon_from_nuc_pos : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;
  typedef prob_gtor_nscodon_from_nuc_pos<FloatType> self_t;

  prob_gtor_nscodon_from_nuc_pos() : base_t(NSCODON::SIZE,PARAM_SIZE) {
    for(unsigned i(1);i<3;++i){
      this->_is_free_block_stop[i*NUC::SIZE-1] = true;
      this->_is_free_block_start[i*NUC::SIZE] = true;
    }
  }

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    nscodon_pdf_2_nuc_pdf_all_pos(this->_distro,this->_param);
  }

  virtual
  void param_update(const bool) {
    for(unsigned i(0);i<CODON::BASE_SIZE;++i){
      pdistro_norm_free_param(this->_param+NUC::SIZE*i,
                              this->_param+NUC::SIZE*(i+1),
                              this->_is_train_param.begin()+NUC::SIZE*i);
    }
    nuc_pdf_all_pos_2_nscodon_pdf(this->_param,this->_distro);
  }

  enum { PARAM_SIZE = NUC::SIZE*CODON::BASE_SIZE };
};




/// \brief frequencies for nsc4 states (pre or post) build from
/// nscodon frequency parameters
///
template <typename FloatType>
struct prob_gtor_trinuc_from_nuc : public prob_gtor<FloatType> {

  typedef prob_gtor<FloatType> base_t;

  typedef prob_gtor_trinuc_from_nuc<FloatType> self_t;

  prob_gtor_trinuc_from_nuc() : base_t(TRINUC::SIZE,PARAM_SIZE) {}

  virtual base_t* clone(const root_gtor_sml_share&) const { return new self_t(*this); }

private:
  virtual
  void get_param_from_distro() const {
    trinuc_pdf_2_nuc_pdf_pos(this->_distro,2,this->_param);
  }

  virtual
  void param_update(const bool) {
    pdistro_norm_free_param(this->_param,
                            this->_param+PARAM_SIZE,
                            this->_is_train_param.begin());
    nuc_pdf_2_trinuc_pdf(this->_param,this->_distro);
  }

  enum { PARAM_SIZE = NUC::SIZE };
};


#endif
