// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: sim_util.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "cat_info.h"
#include "rate_gtor.h"
#include "sim_options.h"
#include "sim_util.h"
#include "subs_ml_model.h"
#include "simple_util.h"
#include "util/math/prob_util.h"
#include "util/math/random_util.h"


const smlfloat SIMULATE_DISCRETE_TIME_UNIT(2.5e-5);


void
get_ancestral_seq(const subs_ml_model& mdl,
                  const unsigned size,
                  const std::vector<unsigned>& site_cats,
                  std::vector<unsigned>& seq){

  const unsigned n_states(mdl.state_size());
  const unsigned n_site_cats(mdl.get_rate_gtor().category_size_site());

  simple_matrix<prob_t> cdf(n_site_cats,n_states);
  for(unsigned i(0);i<n_site_cats;++i){
    pdistro_to_cdf(mdl.get_root_gtor().site_cat_distro(i),cdf.val[i],n_states);
  }

  seq.resize(size);
  for(unsigned i(0);i<size;++i) {
    seq[i] = random_cdf_variate(cdf.val[site_cats[i]],n_states);
  }
}




void
get_sim_site_categories(std::vector<unsigned>& cats,
                        const unsigned size,
                        const rate_gtor_base& rg){

  cats.resize(size);
  const unsigned n_site_cats(rg.category_size_site());
  if(n_site_cats==1){
    for(unsigned i(0);i<size;++i) cats[i] = 0;
  } else {
    prob_t* cat_pdistro(new prob_t[n_site_cats]);
    rg.get_site_category_pdistro(cat_pdistro);
    pdistro_to_cdf_inplace(cat_pdistro,n_site_cats);
    for(unsigned i(0);i<size;++i){ cats[i] = random_cdf_variate(cat_pdistro,n_site_cats); }
    delete [] cat_pdistro;
  }
}




void
sim_site_data_init(const bi_tree_sml_type& tree,
                   const RATE_GTOR_MODEL::index_t rgm,
                   site_data& sd){

  const unsigned n_leaves(tree.n_leaves());

  for(unsigned i(0);i<n_leaves;++i){
    sd.taxid.assignid(tree.leaf[i]->label);
  }
  sd.sm = RATE_GTOR_MODEL::convert_to_site_model(rgm);
  sd.catlabel_2_id[UNASSIGNED_CAT_LABEL] = UNASSIGNED_CAT_ID;
  sd.id_2_catlabel[UNASSIGNED_CAT_ID] = UNASSIGNED_CAT_LABEL;
}
