// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: site_data_fastlup.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "site_data_fastlup.h"
#include "util/general/die.h"

#include <algorithm>



void
site_data_fastlup::
init(unsigned init_len,
     unsigned init_n_orgs,
     unsigned init_n_cats) {

  clear(false);

  base_t::init(init_len,init_n_orgs);

  data = new site_data_fastlup_cell[len];

  n_cats = init_n_cats;
  if(n_cats){
    assigned_cat_count_pool = new unsigned[len*n_cats];
    for(unsigned i(0);i<len;++i){
      data[i].assigned_cat_count = assigned_cat_count_pool+n_cats*i;
    }

    cat_site_mask.resize(n_cats);
    for(unsigned i(0);i<n_cats;++i){ cat_site_mask[i].resize(len,true); }
  }

  for(unsigned i(0);i<len;++i){
    data[i].index=cell_index_pool+i*n_orgs;
  }

}




void
site_data_fastlup::
clear(bool is_clear_base) {
  cat_site_mask.clear();

  if(assigned_cat_count_pool) delete [] assigned_cat_count_pool;
  assigned_cat_count_pool=0;

  if(data) delete [] data; data=0;

   if(is_clear_base) base_t::clear();
}




void
site_data_fastlup::
fix(){

  if(total_count) { die("second fix in site_data_fastlup"); }

  {
    unsigned total_assigned_count(0);
    if(n_cats){
      for(unsigned c(0);c<n_cats;++c){
        for(unsigned i(0);i<len;++i){
          total_assigned_count += data[i].assigned_cat_count[c];
        }
      }
    }

    unsigned total_unassigned_count(0);
    for(unsigned i(0);i<len;++i){
      total_unassigned_count += data[i].unassigned_cat_count;
    }

    is_use_assigned_cats=(total_assigned_count!=0);
    is_use_unassigned_cats=(total_unassigned_count!=0);

    total_count = total_assigned_count+total_unassigned_count;
  }

  /// \todo gcc is using more memory than expected on this sort call,
  /// apparently because there's data in cell_sorter -- debug this or
  /// reduce to a small testcase
  std::sort(data,data+len,cell_sorter);

  {  // create cat masks
    for(unsigned c(0);c<n_cats;++c){
      for(unsigned i(0);i<len;++i){
        if(data[i].unassigned_cat_count || data[i].assigned_cat_count[c]) cat_site_mask[c][i] = false;
      }
    }
  }

  base_t::fix();
}
