// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: subs_ml_model_comex.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __SUBS_ML_MODEL_COMEX_H
#define __SUBS_ML_MODEL_COMEX_H

#include "rate_gtor_options.h"
#include "subs_ml_types.h"

struct subs_ml_model;
struct cat_manager;
struct time_gtor;


/// \brief subs_ml_model component exchange
///
/// present a restricted subset of subs_ml_model functions to model
/// components, such that component communication must move through
/// this structure
///
struct subs_ml_model_comex {
  typedef subs_ml_model_comex self_t;

  subs_ml_model_comex(const subs_ml_model& smlm) : _smlm(smlm) {}

  const cat_manager& get_cat_manager() const;

  RATE_GTOR_MODEL::index_t rate_gtor_model() const;

  unsigned state_size() const;

protected:
  const subs_ml_model& _smlm;
};


struct rate_gtor_sml_share : public subs_ml_model_comex {
  typedef subs_ml_model_comex base_t;

  rate_gtor_sml_share(const subs_ml_model& smlm) : base_t(smlm) {}

  const prob_t* obs_state_distro() const;
  const prob_t* obs_state_distro_assigned_cat(const unsigned c) const;
};


struct root_gtor_sml_share : public subs_ml_model_comex {
  typedef subs_ml_model_comex base_t;

  root_gtor_sml_share(const subs_ml_model& smlm) : base_t(smlm) {}

  void get_obs_state_time_avg_root(prob_t* root_down_prob,
                                   const bool is_assigned_cat,
                                   const unsigned assigned_cat_no) const;

  bool get_root_node_prob(prob_t* root_node_prob,
                          const bool is_leastsq,
                          const bool is_assigned_cat,
                          const unsigned assigned_cat_no) const;

#ifdef BRANCH_SPECIFIC_OBS
  unsigned branch_size() const;

  bool get_root_node_prob_nspround(prob_t* root_node_prob,
                                   const bool is_leastsq,
                                   const bool is_assigned_cat,
                                   const unsigned assigned_cat_no,
                                   prob_t** node_state_prob) const;
#endif
};


#endif
