// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: subs_prob_util.cc 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include "subs_prob_util.h"
#include "subs_prob_util_core.h"


// value below which a branch time is treated as exactly zero:
const smlfloat ZERO_BRANCH_TIME_THRESHOLD(1e-8);



void
get_all_branch_subs_prob(prob_t** branch_prob,
                         const subs_ml_model& mdl,
                         const rates_func_options& opt){

  const unsigned n_branches(mdl.get_time_gtor().branch_size());
  std::vector<bool> is_zero_time_branch(n_branches,false);
  workspace ws[3];

  get_all_branch_subs_prob(branch_prob,is_zero_time_branch,mdl,opt,ws);
}



void
get_all_branch_subs_prob(prob_t** branch_prob,
                         std::vector<bool>& is_zero_time_branch,
                         const subs_ml_model& mdl,
                         const rates_func_options& opt,
                         workspace ws[3]) {

  const unsigned n_branches(mdl.get_time_gtor().branch_size());
  for(unsigned b(0);b<n_branches;++b){
    is_zero_time_branch[b] = (mdl.get_time_gtor().branch_time(b)<=ZERO_BRANCH_TIME_THRESHOLD);
  }

  get_subs_prob(branch_prob,mdl,opt,
                n_branches,mdl.get_time_gtor().branch_times(),is_zero_time_branch,ws);
}



void
get_single_branch_subs_prob(prob_t* branch_prob,
                            const smlfloat time,
                            const subs_ml_model& mdl,
                            const rates_func_options& opt,
                            workspace ws[3]) {

  const bool is_zero(time<=ZERO_BRANCH_TIME_THRESHOLD);

  get_subs_prob(&branch_prob,mdl,opt,
                1,&time,std::vector<bool>(1,is_zero),ws);
}
