// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: subs_prob_util.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __SUBS_PROB_UTIL_H
#define __SUBS_PROB_UTIL_H

#include "rate_gtor_options.h"
#include "subs_ml_types.h"
#include "util/general/workspace.h"

#include <vector>

struct subs_ml_model;

extern const smlfloat ZERO_BRANCH_TIME_THRESHOLD;


/// \brief get the transition prob matrices for every branch in the model
///
void
get_all_branch_subs_prob(prob_t** branch_prob,
                         const subs_ml_model& mdl,
                         const rates_func_options& opt);

/// \brief same as above except this allows the caller to be more
/// efficiently use memory over many calls:
///
void
get_all_branch_subs_prob(prob_t** branch_prob,
                         std::vector<bool>& is_zero_time_branch,
                         const subs_ml_model& mdl,
                         const rates_func_options& opt,
                         workspace ws[3]);

/// \brief get a single transition prob matrix for the model
///
void
get_single_branch_subs_prob(prob_t* branch_prob,
                            const smlfloat time,
                            const subs_ml_model& mdl,
                            const rates_func_options& opt,
                            workspace ws[3]);

#endif
