// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: tree_probs.h 741 2007-08-14 02:39:19Z ctsa $

/// \file

#ifndef __TREE_PROBS_H
#define __TREE_PROBS_H

#include "pprob_cache.h"
#include "simple_util.h"
#include "subs_ml_types.h"
#include "util/general/workspace.h"

#include <vector>

#ifdef DEBUG
//#define PPIT_DEBUG
#endif

struct subs_ml_model;


/// \brief convenience struct to hold branch and root probabilities
/// for a tree
///
struct tree_probs {

  typedef tree_probs self_t;

  tree_probs()
    : n_states(0), n_branches(0), _n_sites(0), _is_site_prob_mode(false) {}

  /// initialize probabilities. **matrices are stored transposed**
  /// so that the 'from' state is the fast index: val = m[from+SIZE*to]
  ///
  void
  init_probs(const subs_ml_model& mdl,
             const unsigned site_category = 0,
             const unsigned group_category = 0,
             const bool is_use_submodel = false,
             const unsigned submodel_no = 0) {
    workspace ws[3];
    init_probs(mdl,site_category,group_category,is_use_submodel,submodel_no,ws);
  }

  /// malloc free version
  void
  init_probs(const subs_ml_model& mdl,
             const unsigned site_category,
             const unsigned group_category,
             const bool is_use_submodel,
             const unsigned submodel_no,
             workspace ws[3]);

  void
  init_site_probs(const subs_ml_model& mdl,
                  const unsigned site_category,
                  const unsigned group_category,
                  const bool is_use_submodel,
                  const unsigned submodel_no,
                  const pprob_cache& left_average,
                  const pprob_cache& right_average,
                  const unsigned n_sites,
                  workspace ws[3]);

  void self_check() const;

  prob_t* root_prob() { return _root_prob.val; }
  prob_t const * root_prob() const { return _root_prob.val; }

  prob_t* branch_prob(const unsigned b) { return _branch_prob.val[b]; }
  prob_t const * branch_prob(const unsigned b) const { return _branch_prob.val[b]; }

  prob_t const * branch_site_prob(const unsigned b,
                                  const unsigned s) const {
    if(_is_site_prob_mode){
      return _timecluster_prob.val[_site_branch_timecluster_map.val[s][b]];
    } else {
      return branch_prob(b);
    }
  }

private:
  void alloc(const unsigned _n_states,
             const unsigned _n_branches);

public:
  unsigned n_states;
  unsigned n_branches;
private:
  unsigned _n_sites;
  simple_init_array<prob_t> _root_prob;
  simple_init_matrix<prob_t> _branch_prob;
  simple_init_matrix<unsigned> _site_branch_cluster_map;
  std::vector<unsigned> _cluster_timecluster_start_map;
  std::vector<bool> _cluster_branch_member;
  simple_init_matrix<unsigned> _site_branch_timecluster_map;
  simple_init_matrix<prob_t> _timecluster_prob;
  bool _is_site_prob_mode;
  simple_init_matrix<prob_t> _exemplar_edge_distro;
public:
  std::vector<bool> is_zero_time_branch;
};


#endif
