// -*- mode: c++; indent-tabs-mode: nil; -*-
//
//
// CodeAxe : phylogenetic analysis and simulation tools
//
//   http://www.phrap.org
//
//
// Copyright 2007 Christopher T Saunders (ctsa@u.washington.edu)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//

// $Id: array_util.hh 741 2007-08-14 02:39:19Z ctsa $

/// \file

#include <numeric>


template <typename ForwardIterator,
          typename InputIterator,
          typename T>
T
array_scale_expectation(ForwardIterator a,
                        const ForwardIterator a_end,
                        InputIterator a_prob,
                        const T target_expect){

  const T expect(std::inner_product(a,a_end,a_prob,T(0)));
  const T scale(target_expect/expect);

  for(;a!=a_end;++a) { *a *= scale; }
  return scale;
}




template <typename ForwardIterator1,
          typename InputIterator,
          typename ForwardIterator2,
          typename T>
T
array_scale_expectation_selected_only(ForwardIterator1 a,
                                      const ForwardIterator1 a_end,
                                      InputIterator a_prob,
                                      ForwardIterator2 a_selected,
                                      const T target_expect){
  {  // if none selected, adjust all...

    bool is_none_selected(true);
    ForwardIterator2 asel(a_selected);
    for(ForwardIterator1 aa(a);aa!=a_end;++aa){
      if(*asel) { is_none_selected=false; break; }
      ++asel;
    }
    if(is_none_selected) {
      return array_scale_expectation(a,a_end,a_prob,target_expect);
    }
  }

  T expect(0),selected_expect(0);

  ForwardIterator2 asel(a_selected);
  for(ForwardIterator1 aa(a);aa!=a_end;++aa){
    const T val((*aa) * (*a_prob));
    expect += val;
    if(*asel) selected_expect += val;
    ++a_prob;
    ++asel;
  }

  const T selected_target_expect(target_expect-expect+selected_expect);
  const T scale(selected_target_expect/selected_expect);

  for(;a!=a_end;++a) {
    if(*a_selected) { *a *= scale; }
    ++a_selected;
  }
  return scale;
}


#if 0
// normalize array by only adjusting selected array members,
// if none are selected, adjust all
//
template <typename ForwardIterator,
          typename ForwardIterator2,
          typename T>
void
array_normalize_selected(ForwardIterator pn,
                         ForwardIterator pn_end,
                         ForwardIterator2 selected,
                         const T norm_val,
                         const T selected_norm_val){

  //  assert( norm_val > std::numeric_limits<T>::epsilon());
  //  assert( selected_norm_val > std::numeric_limits<T>::epsilon());

  {  // if none selected, adjust all...

    bool is_none_selected(true);
    ForwardIterator2 sel(selected);
    for(ForwardIterator p(pn);p!=pn_end;++p){
      if(*sel) { is_none_selected=false; break; }
      ++sel;
    }
    if(is_none_selected) {
      const T scale(T(1)/norm_val);
      for(;pn!=pn_end;++pn) { *pn *= scale; }
      return;
    }
  }

  const T selected_goal_val(T(1)-norm_val+selected_norm_val);
  assert( selected_goal_val != T(0) );

  const T scale(selected_goal_val/selected_norm_val);
  for(;pn!=pn_end;++pn) {
    if(*selected) *pn *= scale;
    ++selected;
  }
}
#endif

